package com.mortals.xhx.feign.device;

import com.alibaba.fastjson.JSONObject;
import com.mortals.framework.common.Rest;
import com.mortals.xhx.common.pdu.RespData;
import com.mortals.xhx.common.pdu.device.DeviceNotifyPdu;
import com.mortals.xhx.common.pdu.device.DevicePdu;
import com.mortals.xhx.feign.IFeign;
import feign.hystrix.FallbackFactory;
import lombok.extern.slf4j.Slf4j;
import org.springframework.cloud.openfeign.FeignClient;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;
/**
 * 设备 Feign接口
 * @author zxfei
 * @date 2022-10-26
 */
@FeignClient(name = "device-manager", path = "/m", fallbackFactory = DeviceFeignFallbackFactory.class)
public interface IDeviceFeign extends IFeign {


    /**
     * 查看设备列表
     *
     * @param devicePdu
     * @return
     */
    @PostMapping(value = "/device/list")
    Rest<RespData<List<DevicePdu>>> list(@RequestBody DevicePdu devicePdu);


    /**
     * 查看设备
     *
     * @param id
     * @return
     */
    @GetMapping(value = "/device/info")
    Rest<DevicePdu> info(@RequestParam(value = "id") Long id);

    /**
     * 删除设备
     *
     * @param ids
     * @return
     */
    @GetMapping(value = "/device/delete")
    Rest<Void> delete(Long[] ids,@RequestHeader("Authorization") String authorization);


    /**
     * 设备保存更新
     *
     * @param devicePdu
     * @return
     */
    @PostMapping(value = "/device/save")
    Rest<RespData<DevicePdu>> save(@RequestBody DevicePdu devicePdu,@RequestHeader("Authorization") String authorization);

    /**
     * 设备启用停用
     *
     * @param devicePdu
     * @return
     */
    @PostMapping(value = "/device/enable")
    String deviceEnable(@RequestBody DevicePdu devicePdu,@RequestHeader("Authorization") String authorization);

    /**
     * 接收第三方平台下发设备消息
     * @param deviceNotifyPdu
     * @return
     */
    @PostMapping(value = "/notify/downMsg")
    Rest<String> downMsg(@RequestBody DeviceNotifyPdu deviceNotifyPdu);

    @PostMapping(value = "/device/importData",consumes= MediaType.MULTIPART_FORM_DATA_VALUE)
    String importData(@RequestPart MultipartFile file,@RequestHeader("Authorization") String authorization,@RequestParam("siteId") Long siteId);
}


@Slf4j
@Component
class DeviceFeignFallbackFactory implements FallbackFactory<IDeviceFeign> {
    @Override
    public IDeviceFeign create(Throwable t) {
        return new IDeviceFeign() {
            @Override
            public Rest<RespData<List<DevicePdu>>> list(DevicePdu devicePdu) {
               return Rest.fail("暂时无法获取设备列表，请稍后再试!");
            }

            @Override
            public Rest<DevicePdu> info(Long id) {
               return Rest.fail("暂时无法获取设备详细，请稍后再试!");
            }

            @Override
            public Rest<Void> delete(Long[] ids, String authorization) {
                return Rest.fail("暂时无法删除设备，请稍后再试!");
            }

            @Override
            public Rest<RespData<DevicePdu>> save(DevicePdu devicePdu, String authorization) {
                return Rest.fail("暂时无法保存设备，请稍后再试!");
            }

            @Override
            public String deviceEnable(DevicePdu devicePdu, String authorization) {
                JSONObject jsonObject = new JSONObject();
                jsonObject.put("code", -1);
                jsonObject.put("msg", "暂时无法启用停用设备，请稍后再试!");
                return jsonObject.toJSONString();
            }

            @Override
            public Rest<String> downMsg(DeviceNotifyPdu deviceNotifyPdu) {
                return Rest.fail("暂时无法接收第三方平台下发设备消息，请稍后再试!");
            }

            @Override
            public String importData(MultipartFile file, String authorization,Long siteId) {
                JSONObject jsonObject = new JSONObject();
                jsonObject.put("code", -1);
                jsonObject.put("msg", "暂时无法导入设备，请稍后再试!");
                return jsonObject.toJSONString();
            }

        };
    }
}








